/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.nokia.messagecounter;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Properties;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.apache.log4j.Logger;
import org.hornetq.api.core.management.ObjectNameBuilder;
import org.hornetq.api.jms.management.JMSQueueControl;

/**
 *
 * @author NOKIA
 */
public class MessageCounter {

    private QueueConnection connection;
    private InitialContext initialContext;
    private QueueConnectionFactory qConnectionFactory;
    private static Queue queue;
    private static final Logger LOGGER = Logger.getLogger(MessageCounter.class);
    private static String JMX_URL = "";
    private JMSQueueControl queueControl;
    private long intervalInSec;
    private int numberOfTimes;

    public MessageCounter() throws Exception {
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws Exception {
        MessageCounter counter = new MessageCounter();

        try {
            if (args.length > 0) {
                if (args[0] != null) {
                    counter.intervalInSec = Long.parseLong(args[0]);
                }

                if (args[1] != null) {
                    counter.numberOfTimes = Integer.parseInt(args[1]);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        counter.startCount();
    }

    private void startCount() throws Exception {
        try {
            intialize();
            int num = 0;
            long nqCount = 0;
            long dqCount = 0;
            long tempNqCount = 0;
            long tempDqCount = 0;
            long messageCount = 0;
            while (num < numberOfTimes) {
                try {
                    LOGGER.info("Waiting for " + intervalInSec + " seconds");
                    Thread.sleep(intervalInSec * 1000);
                } catch (InterruptedException ex) {
                    LOGGER.error("", ex);
                }
                messageCount = queueControl.getMessageCount();
                tempNqCount = queueControl.getMessagesAdded();
                tempDqCount = tempNqCount - queueControl.getDeliveringCount();

                LOGGER.info("After " + intervalInSec + " seconds (total interval: " + intervalInSec * num++ + " seconds):"
                        + "\nNumber of messages in Q: " + messageCount
                        + "\tEqueue count: " + (tempNqCount - nqCount)
                        + "\tDequeue count: " + (tempDqCount - dqCount));
                nqCount = tempNqCount;
                dqCount = tempDqCount;
            }

        } finally {
            if (initialContext != null) {
                initialContext.close();
            }
            if (connection != null) {
                connection.close();
            }
        }
    }

    private void intialize() throws Exception {
        Properties properties = loadProperties(this.getClass().getResourceAsStream("/client-jndi.properties"));
        doLookUps(getInitialContext(properties), properties.getProperty("JNDI_NAME_Q"), properties.getProperty("JNDI_NAME_Q_CONNECTION_FACTORY"));
        connection = qConnectionFactory.createQueueConnection();

        ObjectName on = ObjectNameBuilder.DEFAULT.getJMSQueueObjectName(queue.getQueueName());
        JMXConnector connector = JMXConnectorFactory.connect(new JMXServiceURL(JMX_URL), new HashMap());
        MBeanServerConnection mbsc = connector.getMBeanServerConnection();
        queueControl = (JMSQueueControl) MBeanServerInvocationHandler.newProxyInstance(mbsc, on, JMSQueueControl.class, false);
    }

    private Properties loadProperties(InputStream inputStream) throws IOException {
        Properties properties = new Properties();
        properties.load(inputStream);

        LOGGER.info("JNDI name for Q:" + properties.getProperty("JNDI_NAME_Q"));
        LOGGER.info("JNDI name for QueueConnectionFactory:" + properties.getProperty("JNDI_NAME_Q_CONNECTION_FACTORY"));
        LOGGER.info("JMX_URL: " + (JMX_URL = properties.getProperty("JMX_URL")));
        if(intervalInSec == 0) {
            intervalInSec = Long.parseLong(properties.getProperty("INTERVAL_IN_SEC"));
        }
        LOGGER.info("INTERVAL_IN_SEC: " + intervalInSec);

        if(numberOfTimes == 0) {
            numberOfTimes = numberOfTimes = Integer.parseInt(properties.getProperty("NUMBER_OF_TIMES"));
        }
        LOGGER.info("NUMBER_OF_TIMES: " + numberOfTimes);
        return properties;
    }

    private void doLookUps(Context context, String qJNDIName, String qConFcatoryJNDIName) throws NamingException {
        qConnectionFactory = (QueueConnectionFactory) context.lookup(qConFcatoryJNDIName);
        queue = (Queue) context.lookup(qJNDIName);
        context.close();
    }

    private InitialContext getInitialContext(Properties properties) throws NamingException, FileNotFoundException, IOException {
        return new InitialContext(properties);
    }
}
